import os
import os.path
import sys
import platform
import re
from .__version_file__ import *

oldcwd = os.getcwd()

os.chdir(os.path.dirname(os.path.realpath(__file__)))

base_path = os.path.dirname(os.path.realpath(__file__))

if platform.system() == 'Windows':

    added_dll = None

    # check if safe dll addition is provided (None is to avoid
    # throwing an exception in case attribute is not there).
    if getattr(os, 'add_dll_directory', None):
        added_dll = os.add_dll_directory(base_path + '\\lib')
        os.environ['PATH'] = base_path + '\\lib' + ';' + os.environ['PATH']
    else:
        os.environ['PATH'] = base_path + '\\lib' + ';' + base_path + \
            '\\..\\..;' + os.environ['PATH']


def manual():
    """Returns the full path to the PDF reference manual of the Python
    interface.

    Syntax: xpress.manual()

    Note that only the manual of the Python interface (in PDF format)
    is included in the PyPI and conda package downloaded from these
    repositories; the PDF version of all other Xpress-related
    documentation is contained in the Xpress distribution, and on the
    on-line, HTML format documentation is available on the FICO web
    pages.

    The online documentation includes that for the Xpress Optimizer
    and the Nonlinear solvers and can be found at
    https://www.fico.com/fico-xpress-optimization/docs/latest/overview.html
    """

    import platform

    if platform.system() == 'Windows':
        sep = '\\'
    else:
        sep = '/'

    return base_path + sep + "doc" + sep + "python-interface.pdf"


def examples():
    """
    Returns the full path to the directory containint the examples
    that come with the Python interface.

    Syntax: xpress.examples()

    In the mosel_examples/ subdirectory you will find some of the Mosel
    examples translated into their Python counterpart.

    The online documentation includes that for the Xpress Optimizer and
    the Nonlinear solvers and can be found at
    https://www.fico.com/fico-xpress-optimization/docs/latest/overview.html
    """

    import platform

    if platform.system() == 'Windows':
        sep = '\\'
    else:
        sep = '/'

    return base_path + sep + "examples" + sep


def check_lic_version(path, filename=None):
    """Check license file version against the Xpress library version. If
    the license is more recent, all should be good, otherwise throw up a
    warning (no need to block as the "import xpresslib" will fail).
    """

    global __version__

    import platform

    if platform.system() == 'Windows':
        sep = '\\'
    else:
        sep = '/'

    file = path + sep + filename if filename is not None else path

    try:
        license = open(os.path.abspath(file))
    except:
        if filename is None:
            print('Warning: License file "' + file + '" was not found. ' +
                  'Importing xpress may fail.')
        return None, None

    licmajor, licminor = None, None

    for line in license.readlines():

        s = re.match('.*fico_xpress_release=\"([0-9]*)\\.([0-9]*)\".*', line)

        if s:
            licmajor = int(s.group(1))
            licminor = int(s.group(2))
            break
        elif re.match('EOF.*', line):
            break

    license.close()

    if licmajor is None or \
       licminor is None:
        print('Warning: Could not infer licensed Xpress library version ' +
              'from the license file. Please check that the license file ' +
              'is correct, as there might be issues importing the Xpress ' +
              'module now or in the future.')

    else:
        split_ver = __version__.split('.')
        vermajor = int(split_ver[0])
        verminor = int(split_ver[1])

    if vermajor > licmajor or \
        ((vermajor == licmajor) and
         (verminor > licminor)):
        print('Warning: Your license version ' +
              str(licmajor) + '.' + str(licminor) +
              ' appears to be less recent than the version ' +
              str(__version__) +
              ' of the installed module. This might be cause for a failed ' +
              'import. Please upgrade your license or install an older ' +
              'version of the Xpress module; you can do the latter with\n\n' +
              'pip uninstall xpress\n' +
              'pip install xpress==x.y.z\n\n' +
              '(please check the release history at ' +
              'https://pypi.org/project/xpress/#history) or with conda:\n\n' +
              'conda install xpress=x.y.z\n')

    return licmajor, licminor


__version__ = __version_file__.__version__
__version_library__ = __version_file__.__version_library__


# If the XPRESS environment variable is not set, set it ourselves.
# Then check license number against that of __version__ and give a
# warning if they differ (i.e. if version is more recent than license
# file

if 'XPRESS' not in os.environ.keys():
    os.environ['XPRESS'] = os.path.dirname(os.path.realpath(__file__)) + \
      '/license'
    print(
      "Using the Community license in this session. If you have a full " +
      "Xpress license, first set the XPRESS environment variable to the " +
      "directory containing the license file, xpauth.xpr, and then restart " +
      "Python. If you want to use the FICO Community license and no longer " +
      "want to see this message, set the XPRESS environment variable to\n\n" +
      os.environ['XPRESS'] + '\n')

# Check license version against library version
licensefile = os.environ['XPRESS']

if os.path.isdir(os.path.abspath(licensefile)):
    licmaj, licmin = check_lic_version(licensefile, 'xpauth.xpr')
    licmaj, licmin = check_lic_version(licensefile, 'community-xpauth.xpr')
else:
    licmaj, licmin = check_lic_version(licensefile)


# Try to import the actual library. Once its members/methods are
# imported with "from ... import *" all symbols become those of the
# xpress module.

try:
    from xpresslib import *

except:
    print('Could not import xpress module. ' +
          'We detected that the Xpress Python interface is version ' +
          __version__ + ' and the license file is for version ' +
          str(licmaj) + '.' + str(licmin) + '. Please check that the ' +
          'versions match any installation of the FICO Xpress Optimization ' +
          'suite you may have.\n\n' +
          'If you have a full installation of the FICO Xpress Optimization suite ' +
          'version 8.5.2 or earlier, you should delete all files named ' +
          'xpress*.pyd in the c:\\xpressmp\\bin directory and ' +
          'subdirectories (this may vary depending on where the ' +
          'FICO Xpress installation is located). On Linux or MacOS ' +
          'systems, delete all files named xpress*.so in ' +
          '/opt/xpressmp/lib.')

    raise

else:

    # Check that version in xprs.lib is the same as the one saved in
    # __version__.py

    try:

        split_libver = getversion().split('.')
        split_intver = __version_library__.split('.')

        if split_libver[0] != split_intver[0]:
            print('The Xpress Python API version ' + __version_library__ +
                  ' found the Xpress libraries version ' + getversion() +
                  '. Those versions are incompatible. If you have a local ' +
                  'Xpress installation in addition to the Xpress package in ' +
                  'Python, then please make sure that both Xpress ' +
                  'installations have the same version number.')
        elif __version_library__ != getversion():
            print('Warning: The Xpress Python interface\'s version does not ' +
                  'match that of the Xpress Optimizer library:\n\n' +
                  __version_library__ + '!=' + getversion() +
                  '\n\nWhile the two versions are compatible, you may want '
                  ' to check your installation')

    except InterfaceError as e:

        raise

    except:

        print('Unable to obtain the Xpress Optimizer library version. ' +
              'Please check your installation and/or the license file.')

        raise


# Close import; delete all temporary variables and modules

os.chdir(oldcwd)

if platform.system() == 'Windows':
    if added_dll is not None:
        added_dll.close()
    del added_dll

# Local variables
del split_libver
del split_intver
del oldcwd
del __version_file__

# Local methods
del check_lic_version

# External modules
del os
del sys
del platform
del re
