'''
Transportation problem  with piecewise linear costs
Based on the model Figure 17-3a, Chapter 20 from the book
R.Fourer, DM. Gay, BW Kerninghan AMPL:A modeling language for mathematical programming
'''

import xpress as xp

sources = ['Gary', 'Cleveland', 'Pittsburgh']
destinations = ['Framingham', 'Detroit', 'Lansing', 'Windsor', 'St.Louis', 'Fremont', 'Lafayette']

# Supply from each source
supply = {'Gary': 1400, 'Cleveland': 2600, 'Pittsburgh': 2900}

# Demands at destinations
demand = {'Framingham': 900, 'Detroit': 1200, 'Lansing': 600, 'Windsor': 400,
          'St.Louis': 1700, 'Fremont': 1100, 'Lafayette': 1000}

# Dictionary of breakpoints and slopes for the piecewise linear transportation costs
pw_transp_costs = {('Gary', 'Framingham'): ([(0, 500), (500, 1000), (1000, xp.infinity)], (39, 50, 70)),
                   ('Gary', 'Detroit'): ([(0, 500), (500, 1000), (1000, xp.infinity)], (14, 17, 33)),
                   ('Gary', 'Lansing'): ([(0, 500), (500, 1000), (1000, xp.infinity)], (11, 12, 23)),
                   ('Gary', 'Windsor'): ([(0, 1000), (1000, xp.infinity)], (14, 17)),
                   ('Gary', 'St.Louis'): ([(0, 500), (500, 1000), (1000, xp.infinity)], (16, 23, 40)),
                   ('Gary', 'Fremont'): ([(0, 1000), (1000, xp.infinity)], (82, 98)),
                   ('Gary', 'Lafayette'): ([(0, 500), (500, 1000), (1000, xp.infinity)], (8, 16, 24)),
                   ('Cleveland', 'Framingham'): ([(0, 500), (500, 1000), (1000, xp.infinity)], (27, 37, 47)),
                   ('Cleveland', 'Detroit'): ([(0, 500), (500, 1000), (1000, xp.infinity)], (9, 19, 24)),
                   ('Cleveland', 'Lansing'): ([(0, 500), (500, 1000), (1000, xp.infinity)], (12, 32, 39)),
                   ('Cleveland', 'Windsor'): ([(0, 500), (500, 1000), (1000, xp.infinity)], (9, 14, 21)),
                   ('Cleveland', 'St.Louis'): ([(0, 500), (500, 1000), (1000, xp.infinity)], (26, 36, 47)),
                   ('Cleveland', 'Fremont'): ([(0, 500), (500, 1000), (1000, xp.infinity)], (95, 105, 129)),
                   ('Cleveland', 'Lafayette'): ([(0, 500), (500, 1000), (1000, xp.infinity)], (8, 16, 24)),
                   ('Pittsburgh', 'Framingham'): ([(0, 1000), (1000, xp.infinity)], (24, 34)),
                   ('Pittsburgh', 'Detroit'): ([(0, 1000), (1000, xp.infinity)], (14, 24)),
                   ('Pittsburgh', 'Lansing'): ([(0, 1000), (1000, xp.infinity)], (17, 27)),
                   ('Pittsburgh', 'Windsor'): ([(0, 1000), (1000, xp.infinity)], (13, 23)),
                   ('Pittsburgh', 'St.Louis'): ([(0, xp.infinity)], (28,)),
                   ('Pittsburgh', 'Fremont'): ([(0, 1000), (1000, xp.infinity)], (99, 140)),
                   ('Pittsburgh', 'Lafayette'): ([(0, xp.infinity)], (20,))}

tp = xp.problem()
tp.setprobname('Transportation Problem with PWL Costs')

# variables - transportation amounts
trans = {(i, j): xp.var(vartype=xp.continuous, name='trans_{}_{}'.format(i, j)) for i in sources for j in destinations}
tp.addVariable(trans)

# total shipped from a source must equal supply
for i in sources:
    tp.addConstraint(xp.Sum(trans[(i, j)] for j in destinations) == supply[i])

# total received at a destination must equal demand
for j in destinations:
    tp.addConstraint(xp.Sum(trans[i, j] for i in sources) == demand[j])

# Construct the piecewise linear objective function
obj_expr = None
for key in pw_transp_costs.keys():
    pw_data = pw_transp_costs[key]
    breakpoints = pw_data[0]
    slopes = pw_data[1]
    assert (len(breakpoints) == len(slopes))
    N = range(len(breakpoints))
    cur_expr = {}
    for i in N:
        cur_expr[breakpoints[i]] = slopes[i] * trans[key]
    if obj_expr is None:
        obj_expr = xp.pwl(cur_expr)
    else:
        obj_expr += xp.pwl(cur_expr)

tp.setObjective(obj_expr, sense=xp.minimize)

tp.solve()
